#!/usr/bin/env python
import rospy
import sys
import math
from std_msgs.msg import String
from geometry_msgs.msg import PoseStamped, Point, PoseWithCovarianceStamped, PointStamped, Pose
from actionlib_msgs.msg import GoalID
from unsw_vision_msgs.msg import DetectionList
from trajectory_msgs.msg import JointTrajectory, JointTrajectoryPoint
from control_msgs.msg import JointTrajectoryControllerState

import tf2_ros
import tf2_geometry_msgs

class simple_class:

  def __init__(self):
    
    # tf2 
    self.tfBuffer = tf2_ros.Buffer()
    self.tfListener = tf2_ros.TransformListener(self.tfBuffer)

    self.headGoal = JointTrajectoryPoint()
    self.headGoal.positions = [-1, 0.0]
    self.headGoal.time_from_start = rospy.Duration(secs=5)
    self.head_first_moved = False
    self.goal = Point()
    self.moving = False
    self.stop = False
    self.id = ""
    self.sub = None
    self.last_send_goal_time = None

    rospy.Timer(rospy.Duration(0.5), self.goal_checking)
    self.idsub = rospy.Subscriber("/navigation/person_following/id", String, self.id_callback)
    #self.posesub = rospy.Subscriber("/amcl_pose", PoseWithCovarianceStamped, self.goal_callback, queue_size=1)
    self.stopSub = rospy.Subscriber("/navigation/person_following/stop", String, self.stop_callback)

    self.pub = rospy.Publisher("/move_base_simple/goal", PoseStamped, queue_size=1)
    self.cancel_pub = rospy.Publisher("/move_base/cancel", GoalID, queue_size=1)

    
    self.move_head_pub = rospy.Publisher('/hsrb/head_trajectory_controller/command', JointTrajectory, queue_size=1)
    #self.check_head_sub =  rospy.Subscriber("/hsrb/head_trajectory_controller/state", JointTrajectoryControllerState, self.head_state_cb)
    self.gaze_pub = rospy.Publisher("gaze_point", PointStamped, queue_size=1)
    
    



  # def head_state_cb(self, data:JointTrajectoryControllerState):
  #   # at position 1, so set goal to -1
  #   if data.actual.positions[1] > 0.95 and data.actual.positions[1] < 1.05:
  #     self.headGoal = JointTrajectoryPoint()
  #     self.headGoal.positions = [-1, 0.0]
  #     self.headGoal.time_from_start = rospy.Duration(secs=5)
  #   elif data.actual.positions[1] > -1.05 and data.actual.positions[1] < -0.95:
  #     self.headGoal = JointTrajectoryPoint()
  #     self.headGoal.positions = [1, 0.0]
  #     self.headGoal.time_from_start = rospy.Duration(secs=5)
  #   else:
  #     if self.head_first_moved:
  #       self.headGoal = None


  def id_callback(self, data: String):
    self.id = data.data
    rospy.loginfo(f"I receive id for the person {self.id}")
    if self.sub == None:
      self.stop = False
      self.sub = rospy.Subscriber("/unsw_vision/detections/objects/positions", DetectionList, self.callback)

  
  def callback(self, data: DetectionList):
    # if self.id == "":
    #   return
    rospy.loginfo("following in person follow")
    
    flag = False

    # Find the person with correct id
    dis = 1000
    # for human in data.people:
    #     if human.id == self.id:
    #       point = human.position
    #       flag = True
    #       rospy.loginfo("Found person")
          # break
    for human in data.people:
        if human.distance < dis:
          dis = human.distance
          point = human.position
          flag = True
          rospy.loginfo("Found person")

    # not find the person so move head and end this callback
    if not flag : 
      # if self.headGoal != None:
      #   print("I will move head now")
      #   headPos = JointTrajectory()
      #   headPos.joint_names = ["head_pan_joint", "head_tilt_joint"]
      #   headPos.points = [self.headGoal]
        
      #   rospy.sleep(1)
      #   self.move_head_pub.publish(headPos)
      #   self.head_first_moved = True
      return
    


  
    self.goal = point
    
    # if the point is 0,0 then it is not a valid point
    if point.x == 0 and point.y == 0:
      rospy.loginfo("person point is not usable")
      return

    # apply transform to make the point given into baselink
    robotPoseStamped = PoseStamped()
    robotPoseStamped.pose = Pose()
    
    robotPoseStamped.pose.orientation.w = 1
    robotPoseStamped.header.stamp = rospy.Time.now()
    robotPoseStamped.header.frame_id = 'base_footprint'
    # transform to get the oritation of the robot in the map frame
    try:
        trans = self.tfBuffer.lookup_transform("map","base_footprint",rospy.Time.now(), rospy.Duration(2))
        transformed_point_stamped= tf2_geometry_msgs.do_transform_pose(robotPoseStamped, trans)
        rospy.loginfo(transformed_point_stamped)
    except Exception as e:
        rospy.loginfo("Failed transform point to footprint. {}".format(e))
        return

    
    
    goal = PoseStamped()
    goal.header.stamp = rospy.Time.now()
    goal.header.frame_id = "map"
    # setup goal pose for move_base from the transformed point
    # Do not do self.pose.position = point; pose.position.z=0 as it will change the point.z to 0 and we dont want robot to stare at my feet.
    goal.pose.position.x = point.x
    goal.pose.position.y = point.y
    goal.pose.position.z = 0.0
    # set oritation so robot maintains the rotation

    # Wrong
    # goal.pose.orientation = robotPoseStamped.pose.orientation
    #RIGHT
    goal.pose.orientation =transformed_point_stamped.pose.orientation

    # for gaze, not used now
    gazePoint = PointStamped()
    gazePoint.point = point
    gazePoint.header.stamp = rospy.Time.now()
    gazePoint.header.frame_id = 'map'
    # self.gaze_pub.publish(gazePoint)
    
    # if last goal is sent within 1 seconds do not send a new one to avoid jittering. If the new goal is too close to the original goal, dont go
    if not self.moving or self.last_send_goal_time == None or (rospy.Time.now().to_sec() - self.last_send_goal_time.to_sec() >2 and math.pow(goal.pose.position.x - self.last_nav_goal_pos.x, 2) + math.pow(goal.pose.position.y - self.last_nav_goal_pos.y, 2) < 1):
      rospy.loginfo("I am sending goal")
      self.moving = True
      self.last_send_goal_time = rospy.Time.now()
      self.cancel_goal()
      rospy.sleep(0.5)
      if self.sub == None:
         pass
      else:
        self.last_nav_goal_pos = goal.pose.position
        self.pub.publish(goal)
    else:
      rospy.loginfo("a goal has been sent 1 seconds ago")
    
    
    

  # def goal_callback(self, current_pose: PoseWithCovarianceStamped):
  #   if self.moving == False or self.stop == True: return
  #   dx = math.pow(self.goal.x - current_pose.pose.pose.position.x, 2)
  #   dy = math.pow(self.goal.y - current_pose.pose.pose.position.y, 2)
  #   if (dx + dy) < 1.0*2:
  #     self.moving = False
  #     rospy.loginfo("I am close to the person")
  #     self.cancel_goal()
    
  def goal_checking(self, event):
    if not self.moving or self.stop:
      return
    # apply transform to make the point given into baselink
    robotPoseStamped = PoseStamped()
    robotPoseStamped.pose = Pose()
    
    robotPoseStamped.pose.orientation.w = 1
    robotPoseStamped.header.stamp = rospy.Time.now()
    robotPoseStamped.header.frame_id = 'base_footprint'
    # transform to get the oritation of the robot in the map frame
    try:
        trans = self.tfBuffer.lookup_transform("map","base_footprint",rospy.Time.now(), rospy.Duration(2))
        transformed_pose_stamped= tf2_geometry_msgs.do_transform_pose(robotPoseStamped, trans)
        rospy.loginfo(transformed_pose_stamped)
    except Exception as e:
        rospy.loginfo("Failed transform point to footprint. {}".format(e))
        return
    
    dx = math.pow(self.goal.x - transformed_pose_stamped.pose.pose.position.x, 2)
    dy = math.pow(self.goal.y - transformed_pose_stamped.pose.pose.position.y, 2)
    if (dx + dy) < 1.0*2:
        self.moving = False
        rospy.loginfo("I am close to the person")
        self.cancel_goal()
  def cancel_goal(self):
    msg = GoalID()
    msg.stamp = rospy.Time.now()
    msg.id = "-1"
    self.cancel_pub.publish(msg)
    #self.moving = False

  def stop_callback(self, msg: String):
    self.sub.unregister()
    self.sub = None
    self.cancel_goal()
    self.cancel_goal()
    self.cancel_goal()
    rospy.loginfo("STop person following")
    
    
    self.stop = True
def main(args):
  rospy.init_node('point_follow', anonymous=True)
  obc = simple_class()
  try:
    rospy.spin()
  except KeyboardInterrupt:
    print("Shutting down")

if __name__ == '__main__':
    main(sys.argv)