
CREATE  TABLE "public".agent ( 
	agent_id             integer  NOT NULL GENERATED ALWAYS AS IDENTITY ,
	agent_name           varchar(100)    ,
	height               real    ,
	gender               char(1)    ,
	gender_conf          real    ,
	agent_type           char(1)    ,
	last_seen            timestamp    ,
	CONSTRAINT pk_agent PRIMARY KEY ( agent_id )
 );

CREATE  TABLE "public".agent_attribute_type ( 
	agent_attribute_type_id integer  NOT NULL  ,
	agent_attribute_type varchar(255)    ,
	CONSTRAINT pk_agent_attribute_type PRIMARY KEY ( agent_attribute_type_id )
 );

CREATE  TABLE "public".agent_feature_type ( 
	agent_feature_type_id integer  NOT NULL  ,
	agent_feature_type   varchar(50)    ,
	CONSTRAINT pk_agent_feature_type PRIMARY KEY ( agent_feature_type_id )
 );

CREATE  TABLE "public".agent_speech ( 
	agent_speech_id      integer  NOT NULL  ,
	agent_id             integer  NOT NULL  ,
	phrase               varchar(1000)    ,
	speech_ts            timestamp    ,
	CONSTRAINT pk_agent_speech PRIMARY KEY ( agent_speech_id )
 );

CREATE  TABLE "public".app_info ( 
	db_version           varchar(20)    
 );

CREATE  TABLE "public".object_class_attribute_type ( 
	object_class_attribute_type_id integer  NOT NULL  ,
	object_class_attribute_type varchar(255)    ,
	CONSTRAINT pk_object_class_attribute_type PRIMARY KEY ( object_class_attribute_type_id )
 );

CREATE  TABLE "public".object_group ( 
	object_group_id      integer  NOT NULL  ,
	object_group         varchar(50)    ,
	CONSTRAINT pk_object_group PRIMARY KEY ( object_group_id )
 );

CREATE  TABLE "public".object_label ( 
	object_label_id      integer  NOT NULL  ,
	object_label         varchar(255)    ,
	CONSTRAINT pk_object_label PRIMARY KEY ( object_label_id )
 );

CREATE  TABLE "public".object_relationship_type ( 
	object_relationship_type_id integer  NOT NULL  ,
	object_relationship_type varchar(255)    ,
	CONSTRAINT pk_object_relationship_type PRIMARY KEY ( object_relationship_type_id )
 );

CREATE  TABLE "public".object_shape ( 
	shape_id             integer  NOT NULL  ,
	shape_desc           varchar(255)    ,
	CONSTRAINT pk_object_shape PRIMARY KEY ( shape_id )
 );

CREATE  TABLE "public".object_size ( 
	size_id              integer  NOT NULL  ,
	size_desc            varchar(255)    ,
	CONSTRAINT pk_object_size PRIMARY KEY ( size_id )
 );

CREATE  TABLE "public".region ( 
	region_id            integer  NOT NULL GENERATED BY DEFAULT AS IDENTITY ,
	label                varchar(255)    ,
	polygon              polygon    ,
	neighbors            integer    ,
	CONSTRAINT pk_region PRIMARY KEY ( region_id )
 );

CREATE  TABLE "public".scene ( 
	scene_id             integer  NOT NULL  ,
	scene_ts             timestamp DEFAULT CURRENT_TIMESTAMP   ,
	CONSTRAINT pk_scene PRIMARY KEY ( scene_id )
 );

CREATE  TABLE "public".agent_attribute ( 
	agent_id             integer  NOT NULL  ,
	agent_attribute_type_id integer  NOT NULL  ,
	agent_attribute_value varchar(255)    ,
	CONSTRAINT pk_agent_attribute PRIMARY KEY ( agent_id, agent_attribute_type_id )
 );

CREATE  TABLE "public".agent_feature ( 
	agent_feature_id     integer  NOT NULL GENERATED ALWAYS AS IDENTITY ,
	agent_id             integer  NOT NULL  ,
	agent_feature_type_id integer  NOT NULL  ,
	perspective          varchar(20)    ,
	"encoding"           bytea    ,
	ave_colour           bytea    ,
	image_path           varchar(255)    ,
	last_updated         timestamp DEFAULT CURRENT_TIMESTAMP NOT NULL  ,
	CONSTRAINT pk_pose_encoding PRIMARY KEY ( agent_feature_id )
 );

CREATE  TABLE "public".object_class ( 
	object_class_id      integer  NOT NULL  ,
	name                 varchar(100)    ,
	description          varchar(255)    ,
	pcd_path             varchar(255)    ,
	img_path             varchar(255)    ,
	decay_weight         real    ,
	object_group_id      integer    ,
	region_id            integer    ,
	CONSTRAINT pk_object_class PRIMARY KEY ( object_class_id )
 );

CREATE  TABLE "public".object_class_attribute ( 
	object_class_id      integer  NOT NULL  ,
	object_class_attribute_type_id integer  NOT NULL  ,
	attribute_value      real    ,
	CONSTRAINT pk_object_class_attribute PRIMARY KEY ( object_class_id, object_class_attribute_type_id )
 );

CREATE  TABLE "public".object_class_label ( 
	object_class_id      integer  NOT NULL  ,
	object_label_id      integer  NOT NULL  ,
	CONSTRAINT pk_object_class_label PRIMARY KEY ( object_class_id, object_label_id ),
	CONSTRAINT unq_object_class_label_object_class_id UNIQUE ( object_class_id ) 
 );

CREATE  TABLE "public".point_of_interest ( 
	point_of_interest_id integer  NOT NULL GENERATED BY DEFAULT AS IDENTITY ,
	name                 varchar(100)    ,
	loc_x                real    ,
	loc_y                real    ,
	loc_z                real    ,
	region_id            integer    ,
	CONSTRAINT pk_point_of_interest PRIMARY KEY ( point_of_interest_id )
 );

CREATE  TABLE "public"."object" ( 
	object_id            integer  NOT NULL GENERATED ALWAYS AS IDENTITY ,
	object_class_id      integer  NOT NULL  ,
	class_confidence     real  NOT NULL  ,
	associated_text      varchar(255)    ,
	frame_id             varchar(50)  NOT NULL  ,
	bbox_x               integer    ,
	bbox_y               integer    ,
	bbox_width           integer    ,
	bbox_height          integer    ,
	bbox_cols            integer    ,
	bbox_rows            integer    ,
	loc_x                real    ,
	loc_y                real    ,
	loc_z                real    ,
	loc_confidence       real    ,
	colour               bytea    ,
	texture              bytea    ,
	region_id            integer    ,
	size_id              integer    ,
	shape_id             integer    ,
	created              timestamp  NOT NULL  ,
	last_seen			 timestamp  NULL,
	last_updated         timestamp  NOT NULL  ,
	pcd_path             varchar(255)    ,
	img_path             varchar(255)    ,
	is_current           smallint  NOT NULL  ,
	CONSTRAINT pk_object PRIMARY KEY ( object_id )
 );

CREATE  TABLE "public".object_relationship ( 
	primary_id           integer  NOT NULL  ,
	related_id           integer  NOT NULL  ,
	object_relationship_type_id integer  NOT NULL  ,
	CONSTRAINT pk_object_relationship PRIMARY KEY ( primary_id, related_id, object_relationship_type_id )
 );

CREATE  TABLE "public".robot_state ( 
	robot_state_id       integer  NOT NULL GENERATED ALWAYS AS IDENTITY ,
	frame_id             integer  NOT NULL  ,
	base_pose_x          real    ,
	base_pose_y          real    ,
	base_pose_z          real    ,
	base_pose_w          real    ,
	base_r               real    ,
	base_p               real    ,
	base_yw              real    ,
	head_pose_x          real    ,
	head_pose_y          real    ,
	head_pose_z          real    ,
	head_pose_w          real    ,
	head_r               real    ,
	head_p               real    ,
	head_yw              real    ,
	driveable_state      smallint    ,
	arm_state            smallint    ,
	gripper_state        smallint    ,
	shoulder_state       smallint    ,
	holding_object_id    integer    ,
	created              timestamp  NOT NULL  ,
	is_current           smallint  NOT NULL  ,
	CONSTRAINT pk_robot_state PRIMARY KEY ( robot_state_id )
 );

CREATE  TABLE "public".scene_object ( 
	scene_id             integer  NOT NULL  ,
	object_id            integer  NOT NULL  ,
	CONSTRAINT pk_scene_object PRIMARY KEY ( scene_id, object_id )
 );

CREATE  TABLE "public".agent_object ( 
	agent_id             integer  NOT NULL  ,
	object_id            integer  NOT NULL  ,
	created              timestamp  NOT NULL  ,
	last_updated         timestamp  NOT NULL  ,
	loc_x                real    ,
	loc_y                real    ,
	loc_z                real    ,
	loc_confidence       real    ,
	CONSTRAINT pk_agent_object PRIMARY KEY ( agent_id, object_id )
 );

ALTER TABLE "public".agent_attribute ADD CONSTRAINT fk_agent_attribute_agent FOREIGN KEY ( agent_id ) REFERENCES "public".agent( agent_id );

ALTER TABLE "public".agent_attribute ADD CONSTRAINT fk_agent_attribute FOREIGN KEY ( agent_attribute_type_id ) REFERENCES "public".agent_attribute_type( agent_attribute_type_id );

ALTER TABLE "public".agent_feature ADD CONSTRAINT fk_agent_feature_agent FOREIGN KEY ( agent_id ) REFERENCES "public".agent( agent_id );

ALTER TABLE "public".agent_feature ADD CONSTRAINT fk_agent_feature FOREIGN KEY ( agent_feature_type_id ) REFERENCES "public".agent_feature_type( agent_feature_type_id );

ALTER TABLE "public".agent_object ADD CONSTRAINT fk_agent_object_agent FOREIGN KEY ( agent_id ) REFERENCES "public".agent( agent_id );

ALTER TABLE "public".agent_object ADD CONSTRAINT fk_agent_object_object FOREIGN KEY ( object_id ) REFERENCES "public"."object"( object_id );

ALTER TABLE "public".agent_speech ADD CONSTRAINT fk_agent_speech_agent FOREIGN KEY ( agent_id ) REFERENCES "public".agent( agent_id );

ALTER TABLE "public"."object" ADD CONSTRAINT fk_bbject_object_class FOREIGN KEY ( object_class_id ) REFERENCES "public".object_class( object_class_id );

ALTER TABLE "public"."object" ADD CONSTRAINT fk_object_object_shape FOREIGN KEY ( shape_id ) REFERENCES "public".object_shape( shape_id );

ALTER TABLE "public"."object" ADD CONSTRAINT fk_object_size FOREIGN KEY ( size_id ) REFERENCES "public".object_size( size_id );

ALTER TABLE "public"."object" ADD CONSTRAINT fk_object_location FOREIGN KEY ( region_id ) REFERENCES "public".region( region_id );

ALTER TABLE "public".object_class ADD CONSTRAINT fk_object_class_object_group FOREIGN KEY ( object_group_id ) REFERENCES "public".object_group( object_group_id );

ALTER TABLE "public".object_class ADD CONSTRAINT fk_object_class_region FOREIGN KEY ( region_id ) REFERENCES "public".region( region_id );

ALTER TABLE "public".object_class_attribute ADD CONSTRAINT fk_object_class_attribute_object_class_attribute_type FOREIGN KEY ( object_class_attribute_type_id ) REFERENCES "public".object_class_attribute_type( object_class_attribute_type_id );

ALTER TABLE "public".object_class_attribute ADD CONSTRAINT fk_object_class_attribute_object_class FOREIGN KEY ( object_class_id ) REFERENCES "public".object_class( object_class_id );

ALTER TABLE "public".object_class_label ADD CONSTRAINT fk_object_class_label_object_class FOREIGN KEY ( object_class_id ) REFERENCES "public".object_class( object_class_id );

ALTER TABLE "public".object_class_label ADD CONSTRAINT fk_object_class_label_object_label FOREIGN KEY ( object_label_id ) REFERENCES "public".object_label( object_label_id );

ALTER TABLE "public".object_relationship ADD CONSTRAINT fk_object_rel_1 FOREIGN KEY ( primary_id ) REFERENCES "public"."object"( object_id );

ALTER TABLE "public".object_relationship ADD CONSTRAINT fk_object_rel_2 FOREIGN KEY ( related_id ) REFERENCES "public"."object"( object_id );

ALTER TABLE "public".object_relationship ADD CONSTRAINT fk_object_rel_type FOREIGN KEY ( object_relationship_type_id ) REFERENCES "public".object_relationship_type( object_relationship_type_id );

ALTER TABLE "public".point_of_interest ADD CONSTRAINT fk_point_of_interest_region FOREIGN KEY ( region_id ) REFERENCES "public".region( region_id );

ALTER TABLE "public".robot_state ADD CONSTRAINT fk_robot_state_object FOREIGN KEY ( holding_object_id ) REFERENCES "public"."object"( object_id );

ALTER TABLE "public".scene_object ADD CONSTRAINT fk_scene_object_object FOREIGN KEY ( object_id ) REFERENCES "public"."object"( object_id );

ALTER TABLE "public".scene_object ADD CONSTRAINT fk_scene_object_scene FOREIGN KEY ( scene_id ) REFERENCES "public".scene( scene_id );

CREATE OR REPLACE FUNCTION public.update_object_region()
 RETURNS trigger
 LANGUAGE plpgsql
AS $function$
begin
    --raise notice 'hi: %', NEW;
    -- lookup correct polygon and store this as the region for the object
    NEW.region_id = (SELECT r.region_id FROM region r WHERE point (NEW.loc_x, NEW.loc_y) <@ r.polygon FETCH FIRST ROW ONLY);
    RETURN NEW;
end;
$function$
;

CREATE OR REPLACE VIEW "public".v_cur_object AS SELECT o.object_id, o.object_class_id, oc.name, o.class_confidence, o.frame_id, o.bbox_x, o.bbox_y, o.bbox_width, o.bbox_height, o.bbox_cols, 
o.bbox_rows, o.loc_x, o.loc_y, o.loc_z, o.loc_confidence, o.colour, o.texture, o.region_id, r.label, o.size_id, sz.size_desc, o.shape_id, sh.shape_desc,
o.created, o.last_seen, o.last_updated
FROM public.object o inner join public.object_class oc on o.object_class_id = oc.object_class_id
left outer join public.region r on o.region_id = r.region_id
left outer join public.object_size sz on o.size_id = sz.size_id
left outer join public.object_shape sh on o.shape_id = sh.shape_id
WHERE is_current = 1;

CREATE OR REPLACE VIEW "public".v_scene AS SELECT so.scene_id, s.scene_ts, so.object_id, oc.name
FROM public.scene_object so 
inner join public.scene s on so.scene_id = s.scene_id
inner join public.object o on so.object_id = o.object_id
left outer join public.object_class oc on o.object_class_id = oc.object_class_id;

CREATE TRIGGER object_location_change BEFORE INSERT OR UPDATE ON public.object FOR EACH ROW EXECUTE FUNCTION update_object_region();

COMMENT ON COLUMN "public".agent.agent_type IS 'E''E''P = Person\\
R = Robot''''';

COMMENT ON TABLE "public".agent_object IS 'This table contains the objects that the agent has seen. It forms the world model from the agent perspective';

COMMENT ON COLUMN "public".agent_object.loc_confidence IS 'The location confidence, i.e. the confidence that the agent still know the object is in this location.';

COMMENT ON VIEW "public".v_cur_object IS 'This view contains the objects that are current in the world model, i.e. have not been deleted.';
