CREATE SCHEMA IF NOT EXISTS "public";

CREATE  TABLE "public".action_type ( 
	action_type_id       integer  NOT NULL  ,
	action_type          varchar(255)    ,
	CONSTRAINT pk_action_type PRIMARY KEY ( action_type_id ),
	CONSTRAINT unq_action_type_action_type UNIQUE ( action_type )
 );

CREATE  TABLE "public".agent ( 
	agent_id             integer  NOT NULL GENERATED ALWAYS AS IDENTITY ,
	tracking_id          integer    ,
	agent_name           varchar(100)    ,
	height               real    ,
	gender               char(1)    ,
	gender_conf          real    ,
	age                  integer    ,
	age_conf             real    ,
	agent_type           char(1)    ,
	img_path             varchar(255)    ,
	created              timestamp(6)    ,
	CONSTRAINT pk_agent PRIMARY KEY ( agent_id )
 );

CREATE  TABLE "public".agent_attribute_type ( 
	agent_attribute_type_id integer  NOT NULL  ,
	agent_attribute_type varchar(255)    ,
	CONSTRAINT pk_agent_attribute_type PRIMARY KEY ( agent_attribute_type_id )
 );

CREATE  TABLE "public".agent_feature_type ( 
	agent_feature_type_id integer  NOT NULL  ,
	agent_feature_type   varchar(50)    ,
	CONSTRAINT pk_agent_feature_type PRIMARY KEY ( agent_feature_type_id )
 );

CREATE  TABLE "public".agent_speech ( 
	agent_speech_id      integer  NOT NULL  ,
	agent_id             integer  NOT NULL  ,
	phrase               varchar(1000)    ,
	speech_ts            timestamp(6)    ,
	CONSTRAINT pk_agent_speech PRIMARY KEY ( agent_speech_id )
 );

CREATE  TABLE "public".app_info ( 
	db_version           varchar(20)    
 );

CREATE  TABLE "public".object_class_attribute_type ( 
	object_class_attribute_type_id integer  NOT NULL  ,
	object_class_attribute_type varchar(255)    ,
	CONSTRAINT pk_object_class_attribute_type PRIMARY KEY ( object_class_attribute_type_id )
 );

CREATE  TABLE "public".object_group ( 
	object_group_id      integer  NOT NULL  ,
	object_group         varchar(50)    ,
	CONSTRAINT pk_object_group PRIMARY KEY ( object_group_id )
 );

CREATE  TABLE "public".object_label ( 
	object_label_id      integer  NOT NULL  ,
	object_label         varchar(255)    ,
	CONSTRAINT pk_object_label PRIMARY KEY ( object_label_id )
 );

CREATE  TABLE "public".object_relationship_type ( 
	object_relationship_type_id integer  NOT NULL  ,
	object_relationship_type varchar(255)    ,
	CONSTRAINT pk_object_relationship_type PRIMARY KEY ( object_relationship_type_id )
 );

CREATE  TABLE "public".object_shape ( 
	shape_id             integer  NOT NULL  ,
	shape_desc           varchar(255)    ,
	CONSTRAINT pk_object_shape PRIMARY KEY ( shape_id )
 );

CREATE  TABLE "public".object_size ( 
	size_id              integer  NOT NULL  ,
	size_desc            varchar(255)    ,
	CONSTRAINT pk_object_size PRIMARY KEY ( size_id )
 );

CREATE  TABLE "public".region ( 
	region_id            integer  NOT NULL GENERATED BY DEFAULT AS IDENTITY ,
	label                varchar(255)    ,
	polygon              polygon    ,
	neighbors            integer    ,
	CONSTRAINT pk_region PRIMARY KEY ( region_id )
 );

CREATE  TABLE "public".robot_goals ( 
	robot_goal_id        integer  NOT NULL GENERATED BY DEFAULT AS IDENTITY  ,
	agent_id             integer    ,
	goal_ts              timestamp    ,
	goal_state           char(1)    ,
	action_type_id       integer  NOT NULL  ,
	action_descriptor    varchar(255)    ,
	CONSTRAINT pk_robot_goals PRIMARY KEY ( robot_goal_id )
 );

CREATE  TABLE "public".scene ( 
	scene_id             integer  NOT NULL  ,
	scene_ts             timestamp(6) DEFAULT CURRENT_TIMESTAMP   ,
	CONSTRAINT pk_scene PRIMARY KEY ( scene_id )
 );

CREATE  TABLE "public".agent_attribute ( 
	agent_id             integer  NOT NULL  ,
	agent_attribute_type_id integer  NOT NULL  ,
	agent_attribute_value varchar(255)    ,
	CONSTRAINT pk_agent_attribute PRIMARY KEY ( agent_id, agent_attribute_type_id )
 );

CREATE  TABLE "public".agent_feature ( 
	agent_id             integer  NOT NULL  ,
	agent_feature_type_id integer  NOT NULL  ,
	perspective          varchar(20)  NOT NULL  ,
	"encoding"           double precision[]    ,
	img_path             varchar(255)    ,
	last_updated         timestamp(6) DEFAULT CURRENT_TIMESTAMP NOT NULL  ,
	CONSTRAINT pk_agent_feature PRIMARY KEY ( agent_id, agent_feature_type_id, perspective )
 );

CREATE  TABLE "public".agent_location ( 
	agent_id             integer  NOT NULL  ,
	loc_x                real    ,
	loc_y                real    ,
	loc_z                real    ,
	region_id            integer    ,
	seen_time            timestamp DEFAULT CURRENT_TIMESTAMP   ,
	CONSTRAINT pk_agent_location UNIQUE ( agent_id, seen_time ) 
 );

CREATE  TABLE "public".object_class ( 
	object_class_id      integer  NOT NULL  ,
	name                 varchar(100)    ,
	description          varchar(255)    ,
	pcd_path             varchar(255)    ,
	img_path             varchar(255)    ,
	decay_weight         real    ,
	object_group_id      integer  NOT NULL  ,
	region_id            integer    ,
	CONSTRAINT pk_object_class PRIMARY KEY ( object_class_id )
 );

CREATE  TABLE "public".object_class_attribute ( 
	object_class_id      integer  NOT NULL  ,
	object_class_attribute_type_id integer  NOT NULL  ,
	attribute_value      real    ,
	CONSTRAINT pk_object_class_attribute PRIMARY KEY ( object_class_id, object_class_attribute_type_id )
 );

CREATE  TABLE "public".object_class_label ( 
	object_class_id      integer  NOT NULL  ,
	object_label_id      integer  NOT NULL  ,
	CONSTRAINT pk_object_class_label PRIMARY KEY ( object_class_id, object_label_id ),
	CONSTRAINT unq_object_class_label_object_class_id UNIQUE ( object_class_id ) 
 );

CREATE  TABLE "public".object_sub_class ( 
	object_sub_class_id  integer  NOT NULL  ,
	object_class_id      integer  NOT NULL  ,
	name                 varchar(100)    ,
	object_group_id      integer  NOT NULL  ,
	CONSTRAINT pk_object_sub_class PRIMARY KEY ( object_sub_class_id )
 );

CREATE  TABLE "public".object_sub_class_features ( 
	object_sub_class_feature_id integer  NOT NULL  ,
	object_sub_class_id  integer  NOT NULL  ,
	"encoding"           double precision[]    ,
	CONSTRAINT pk_object_sub_class_features PRIMARY KEY ( object_sub_class_feature_id )
 );

CREATE  TABLE "public".point_of_interest ( 
	point_of_interest_id integer  NOT NULL GENERATED BY DEFAULT AS IDENTITY ,
	name                 varchar(100)    ,
	loc_x                real    ,
	loc_y                real    ,
	loc_z                real    ,
	region_id            integer    ,
	CONSTRAINT pk_point_of_interest PRIMARY KEY ( point_of_interest_id )
 );

CREATE  TABLE "public"."object" ( 
	object_id            integer  NOT NULL GENERATED ALWAYS AS IDENTITY ,
	object_class_id      integer  NOT NULL  ,
	class_confidence     real  NOT NULL  ,
	associated_text      varchar(255)    ,
	frame_id             varchar(50)  NOT NULL  ,
	bbox_x               integer    ,
	bbox_y               integer    ,
	bbox_width           integer    ,
	bbox_height          integer    ,
	bbox_cols            integer    ,
	bbox_rows            integer    ,
	loc_x                real    ,
	loc_y                real    ,
	loc_z                real    ,
	loc_confidence       real    ,
	mean_colour          double precision[]    ,
	texture              double precision[]    ,
	region_id            integer    ,
	size_id              integer    ,
	shape_id             integer    ,
	created              timestamp(6)  NOT NULL  ,
	last_seen            timestamp(6)    ,
	last_updated         timestamp(6)  NOT NULL  ,
	pcd_path             varchar(255)    ,
	img_path             varchar(255)    ,
	is_current           smallint  NOT NULL  ,
	is_removed           smallint    ,
	CONSTRAINT pk_object PRIMARY KEY ( object_id )
 );

CREATE  TABLE "public".object_relationship ( 
	primary_id           integer  NOT NULL  ,
	related_id           integer  NOT NULL  ,
	object_relationship_type_id integer  NOT NULL  ,
	CONSTRAINT pk_object_relationship PRIMARY KEY ( primary_id, related_id, object_relationship_type_id )
 );

CREATE  TABLE "public".robot_state ( 
	robot_state_id       integer  NOT NULL GENERATED ALWAYS AS IDENTITY ,
	frame_id             varchar(10)  NOT NULL  ,
	base_pose_x          real    ,
	base_pose_y          real    ,
	base_pose_z          real    ,
	region_id            integer    ,
	base_r               real    ,
	base_p               real    ,
	base_yw              real    ,
	head_pose_z          real    ,
	head_r               real    ,
	head_p               real    ,
	head_yw              real    ,
	end_effector_x		 real	 ,
	end_effector_y       real    ,
	end_effector_z       real    ,
	end_effector_r       real    ,
	end_effector_p       real    ,
	end_effector_yw      real    ,
	driveable_state      boolean    ,
	is_wrist_pressed     boolean    ,
	arm_state            real    ,
	gripper_closed       boolean    ,
	listening_state      smallint    ,
	speaking_state       smallint    ,
	battery_status       real    ,
	holding_object_id    integer    ,
	created              timestamp(6)  NOT NULL  ,
	is_current           smallint  NOT NULL  ,
	CONSTRAINT pk_robot_state PRIMARY KEY ( robot_state_id )
 );

CREATE  TABLE "public".scene_object ( 
	scene_id             integer  NOT NULL  ,
	object_id            integer  NOT NULL  ,
	CONSTRAINT pk_scene_object PRIMARY KEY ( scene_id, object_id )
 );

CREATE  TABLE "public".agent_object ( 
	agent_object_id      integer  NOT NULL GENERATED BY DEFAULT AS IDENTITY  ,
	agent_id             integer  NOT NULL  ,
	object_id            integer    ,
	object_class_id      integer    ,
	region_id            integer    ,
	loc_x                real    ,
	loc_y                real    ,
	loc_z                real    ,
	loc_confidence       real    ,
	created              timestamp(6)  NOT NULL  ,
	last_updated         timestamp(6)  NOT NULL  ,
	last_seen            timestamp(6)  NOT NULL  ,
	CONSTRAINT pk_agent_object PRIMARY KEY ( agent_object_id )
 );

ALTER TABLE "public".agent_attribute ADD CONSTRAINT fk_agent_attribute_agent FOREIGN KEY ( agent_id ) REFERENCES "public".agent( agent_id );

ALTER TABLE "public".agent_attribute ADD CONSTRAINT fk_agent_attribute FOREIGN KEY ( agent_attribute_type_id ) REFERENCES "public".agent_attribute_type( agent_attribute_type_id );

ALTER TABLE "public".agent_feature ADD CONSTRAINT fk_agent_feature_agent FOREIGN KEY ( agent_id ) REFERENCES "public".agent( agent_id );

ALTER TABLE "public".agent_feature ADD CONSTRAINT fk_agent_feature FOREIGN KEY ( agent_feature_type_id ) REFERENCES "public".agent_feature_type( agent_feature_type_id );

ALTER TABLE "public".agent_location ADD CONSTRAINT fk_agent_location_agent FOREIGN KEY ( agent_id ) REFERENCES "public".agent( agent_id );

ALTER TABLE "public".agent_location ADD CONSTRAINT fk_agent_location_region FOREIGN KEY ( region_id ) REFERENCES "public".region( region_id );

ALTER TABLE "public".agent_object ADD CONSTRAINT fk_agent_object_agent FOREIGN KEY ( agent_id ) REFERENCES "public".agent( agent_id );

ALTER TABLE "public".agent_object ADD CONSTRAINT fk_agent_object_object FOREIGN KEY ( object_id ) REFERENCES "public"."object"( object_id );

ALTER TABLE "public".agent_object ADD CONSTRAINT fk_agent_object_object_class FOREIGN KEY ( object_class_id ) REFERENCES "public".object_class( object_class_id );

ALTER TABLE "public".agent_object ADD CONSTRAINT fk_agent_object_region FOREIGN KEY ( region_id ) REFERENCES "public".region( region_id );

ALTER TABLE "public".agent_speech ADD CONSTRAINT fk_agent_speech_agent FOREIGN KEY ( agent_id ) REFERENCES "public".agent( agent_id );

ALTER TABLE "public"."object" ADD CONSTRAINT fk_bbject_object_class FOREIGN KEY ( object_class_id ) REFERENCES "public".object_class( object_class_id );

ALTER TABLE "public"."object" ADD CONSTRAINT fk_object_object_shape FOREIGN KEY ( shape_id ) REFERENCES "public".object_shape( shape_id );

ALTER TABLE "public"."object" ADD CONSTRAINT fk_object_size FOREIGN KEY ( size_id ) REFERENCES "public".object_size( size_id );

ALTER TABLE "public"."object" ADD CONSTRAINT fk_object_location FOREIGN KEY ( region_id ) REFERENCES "public".region( region_id );

ALTER TABLE "public".object_class ADD CONSTRAINT fk_object_class_object_group FOREIGN KEY ( object_group_id ) REFERENCES "public".object_group( object_group_id );

ALTER TABLE "public".object_class ADD CONSTRAINT fk_object_class_region FOREIGN KEY ( region_id ) REFERENCES "public".region( region_id );

ALTER TABLE "public".object_class_attribute ADD CONSTRAINT fk_object_class_attribute_object_class_attribute_type FOREIGN KEY ( object_class_attribute_type_id ) REFERENCES "public".object_class_attribute_type( object_class_attribute_type_id );

ALTER TABLE "public".object_class_attribute ADD CONSTRAINT fk_object_class_attribute_object_class FOREIGN KEY ( object_class_id ) REFERENCES "public".object_class( object_class_id );

ALTER TABLE "public".object_class_label ADD CONSTRAINT fk_object_class_label_object_class FOREIGN KEY ( object_class_id ) REFERENCES "public".object_class( object_class_id );

ALTER TABLE "public".object_class_label ADD CONSTRAINT fk_object_class_label_object_label FOREIGN KEY ( object_label_id ) REFERENCES "public".object_label( object_label_id );

ALTER TABLE "public".object_relationship ADD CONSTRAINT fk_object_rel_1 FOREIGN KEY ( primary_id ) REFERENCES "public"."object"( object_id );

ALTER TABLE "public".object_relationship ADD CONSTRAINT fk_object_rel_2 FOREIGN KEY ( related_id ) REFERENCES "public"."object"( object_id );

ALTER TABLE "public".object_relationship ADD CONSTRAINT fk_object_rel_type FOREIGN KEY ( object_relationship_type_id ) REFERENCES "public".object_relationship_type( object_relationship_type_id );

ALTER TABLE "public".object_sub_class ADD CONSTRAINT fk_object_sub_class FOREIGN KEY ( object_class_id ) REFERENCES "public".object_class( object_class_id );

ALTER TABLE "public".object_sub_class ADD CONSTRAINT fk_object_sub_class_object_group FOREIGN KEY ( object_group_id ) REFERENCES "public".object_group( object_group_id );

ALTER TABLE "public".object_sub_class_features ADD CONSTRAINT fk_object_sub_class_features_object_sub_class FOREIGN KEY ( object_sub_class_id ) REFERENCES "public".object_sub_class( object_sub_class_id );

ALTER TABLE "public".point_of_interest ADD CONSTRAINT fk_point_of_interest_region FOREIGN KEY ( region_id ) REFERENCES "public".region( region_id );

ALTER TABLE "public".robot_goals ADD CONSTRAINT fk_robot_goals_action_type FOREIGN KEY ( action_type_id ) REFERENCES "public".action_type( action_type_id );

ALTER TABLE "public".robot_goals ADD CONSTRAINT fk_robot_goals_agent FOREIGN KEY ( agent_id ) REFERENCES "public".agent( agent_id );

ALTER TABLE "public".robot_state ADD CONSTRAINT fk_robot_state_object FOREIGN KEY ( holding_object_id ) REFERENCES "public"."object"( object_id );

ALTER TABLE "public".robot_state ADD CONSTRAINT fk_robot_state_region FOREIGN KEY ( region_id ) REFERENCES "public".region( region_id );

ALTER TABLE "public".scene_object ADD CONSTRAINT fk_scene_object_object FOREIGN KEY ( object_id ) REFERENCES "public"."object"( object_id );

ALTER TABLE "public".scene_object ADD CONSTRAINT fk_scene_object_scene FOREIGN KEY ( scene_id ) REFERENCES "public".scene( scene_id );

CREATE OR REPLACE VIEW "public".v_cur_objects AS SELECT o.object_id, o.object_class_id, oc.name, o.class_confidence, o.frame_id, o.bbox_x, o.bbox_y, o.bbox_width, o.bbox_height, o.bbox_cols, 
o.bbox_rows, o.loc_x, o.loc_y, o.loc_z, o.loc_confidence, o.mean_colour, o.texture, o.region_id, r.label, o.size_id, sz.size_desc, o.shape_id, sh.shape_desc,
o.created, o.last_updated, o.last_seen
FROM public.object o inner join public.object_class oc on o.object_class_id = oc.object_class_id
left outer join public.region r on o.region_id = r.region_id
left outer join public.object_size sz on o.size_id = sz.size_id
left outer join public.object_shape sh on o.shape_id = sh.shape_id
WHERE is_current = 1;

CREATE OR REPLACE VIEW "public".v_scene AS 
SELECT s.scene_id, s.scene_ts, so.object_id, oc.name
FROM scene s inner join scene_object so on s.scene_id = so.scene_id 
inner join object o on so.object_id = o.object_id 
inner join object_class oc on o.object_class_id = oc.object_class_id;

CREATE OR REPLACE FUNCTION public.update_object_region()
 RETURNS trigger
 LANGUAGE plpgsql
AS $function$
begin
    --raise notice 'hi: %', NEW;
    -- lookup correct polygon and store this as the region for the object
    NEW.region_id = (SELECT r.region_id FROM region r WHERE point (NEW.loc_x, NEW.loc_y) <@ r.polygon FETCH FIRST ROW ONLY);
    RETURN NEW;
end;
$function$
;

CREATE OR REPLACE FUNCTION public.can_see(object_class TEXT) RETURNS BOOLEAN AS $$
BEGIN
    RETURN EXISTS (SELECT 1 FROM v_recent_scene rs WHERE rs.obj_class = object_class);
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE VIEW public.v_recent_scene
 AS
 SELECT DISTINCT so.object_id,
    obj.name AS object_class
   FROM scene_object so
     JOIN v_cur_objects obj ON so.object_id = obj.object_id
  WHERE so.scene_id >= ( SELECT max(scene.scene_id) - 3 FROM scene);

ALTER TABLE public.v_recent_scene
    OWNER TO postgres;
COMMENT ON VIEW public.v_recent_scene
    IS 'Returns the distinct objects and object class in the most recent 3 scenes';


CREATE TRIGGER object_location_change BEFORE INSERT OR UPDATE ON public.object FOR EACH ROW EXECUTE FUNCTION update_object_region();

COMMENT ON COLUMN "public".agent.agent_type IS 'P = Person;
R = Robot''';

COMMENT ON COLUMN "public".robot_goals.goal_state IS 'n = not started; s = success; i = in progress; f = failed;';

COMMENT ON TABLE "public".object_sub_class_features IS 'The features related to the object sub class, e.g. a can ok coke will have certain features such as the text on the can, the smoothness etc. This table will contain features for a specific object class that can be used to distinguish between specific instances of objects, e.g. coke and sprite are both cans but have different text and images';

COMMENT ON COLUMN "public".robot_state.listening_state IS '0 = not listening; 1 = listening';

COMMENT ON COLUMN "public".robot_state.speaking_state IS '0 = not speaking; 1 = speaking';

COMMENT ON TABLE "public".agent_object IS 'This table contains the objects that the agent has seen. It forms the world model from the agent perspective';

COMMENT ON COLUMN "public".agent_object.object_class_id IS 'If the instance of the object is not known then the class of object is sufficient';

COMMENT ON COLUMN "public".agent_object.loc_confidence IS 'The location confidence, i.e. the confidence that the agent still know the object is in this location.';
